/**
 * MCS Media Computer Software
 * Copyright 2018 by Wilfried Klaas
 * Project: SPSEmulator
 * File: Not.java
 * EMail: W.Klaas@gmx.de
 * Created: 25.11.2018 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps.mnemonic;

import java.util.EnumSet;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

/**
 * MOV X, Y: setting register X to value of register Y (X=Y)
 * 
 * @author wklaa_000
 *
 */
public class MOV extends AbstractMnemonic implements Mnemonic {

  private enum MOVE_VALUE {
    MOV_BA("A", "B", 0x51), MOV_CA("A", "C", 0x52), MOV_DA("A", "D", 0x53), MOV_EA("A", "E", 0x5d), MOV_FA("A", "F",
        0x5e), MOV_AB("B", "A",
            0x61), MOV_AC("C", "A", 0x62), MOV_AD("D", "A", 0x63), MOV_AE("E", "A", 0x6d), MOV_AF("F", "A", 0x6e);

    String source;
    String dest;
    int value;

    MOVE_VALUE(String source, String dest, int value) {
      this.source = source;
      this.dest = dest;
      this.value = value;
    }

  };

  byte output;
  MOVE_VALUE actual;

  public MOV(String line) throws SyntaxError {
    super(line);
    output = 0;
  }

  @Override
  public void checkArgument() throws SyntaxError {
    String argument = getArgument().trim();
    if (StringUtils.isEmpty(argument)) {
      throw new SyntaxError(getLineNumber(),
          String.format("missing arguments for %s.", this.getClass().getSimpleName()));
    }
    if (argument.indexOf(",") < 0) {
      throw new SyntaxError(getLineNumber(),
          String.format("missing arguments for %s.", this.getClass().getSimpleName()));
    }
    int value = -1;
    argument = argument.replace(",", "");
    argument = argument.replace(" ", "");

    actual = getActualMoveValue(argument);
    if (actual != null) {
      value = actual.value;
    }

    if (value < 0) {
      throw new IllegalArgument(getLineNumber(),
          String.format("argument %s is not in enum for %s.", getArgument(), this.getClass().getSimpleName()));
    }

    output = (byte) (value & 0xFF);
  }

  private MOVE_VALUE getActualMoveValue(String argument) throws IllegalArgument {
    try {
      MOVE_VALUE moveValue = MOVE_VALUE.valueOf(String.format("MOV_%S", argument.trim()));
      if (moveValue != null) {
        return moveValue;
      }
    } catch (IllegalArgumentException e) {
      throw new IllegalArgument(getLineNumber(),
          String.format("argument %s is not in enum for %s.", getArgument(), this.getClass().getSimpleName()));
    }
    return null;
  }

  @Override
  public int getByte() {
    return output;
  }

  @Override
  public boolean isUsingPage() {
    return false;
  }

  @Override
  public boolean hasArgument() {
    return true;
  }

  @Override
  public Set<HARDWARE> allowedHardware() {
    switch (actual) {
    case MOV_EA:
    case MOV_FA:
    case MOV_AE:
    case MOV_AF:
      return EnumSet.of(HARDWARE.ARDUINOSPS, HARDWARE.TINYSPS);
    default:
      return super.allowedHardware();
    }
  }
}
